@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -90,26 +90,6 @@ gint at_util_call_compare(gconstpointer a, gconstpointer b)
 	return 0;
 }
 
-unsigned int at_util_alloc_next_id(unsigned int *id_list)
-{
-	unsigned int i;
-
-	for (i = 1; i < sizeof(unsigned int) * 8; i++) {
-		if (*id_list & (1 << i))
-			continue;
-
-		*id_list |= (1 << i);
-		return i;
-	}
-
-	return 0;
-}
-
-void at_util_release_id(unsigned int *id_list, unsigned int id)
-{
-	*id_list &= ~(1 << id);
-}
-
 GSList *at_util_parse_clcc(GAtResult *result)
 {
 	GAtResultIter iter;
@@ -167,3 +147,99 @@ GSList *at_util_parse_clcc(GAtResult *result)
 	return l;
 }
 
+gboolean at_util_parse_reg_unsolicited(GAtResult *result, const char *prefix,
+					int *status,
+					int *lac, int *ci, int *tech)
+{
+	GAtResultIter iter;
+	int s;
+	int l = -1, c = -1, t = -1;
+	const char *str;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (g_at_result_iter_next(&iter, prefix) == FALSE)
+		return FALSE;
+
+	if (g_at_result_iter_next_number(&iter, &s) == FALSE)
+		return FALSE;
+
+	if (g_at_result_iter_next_string(&iter, &str) == TRUE)
+		l = strtol(str, NULL, 16);
+	else
+		goto out;
+
+	if (g_at_result_iter_next_string(&iter, &str) == TRUE)
+		c = strtol(str, NULL, 16);
+	else
+		goto out;
+
+	g_at_result_iter_next_number(&iter, &t);
+
+out:
+	if (status)
+		*status = s;
+
+	if (lac)
+		*lac = l;
+
+	if (ci)
+		*ci = c;
+
+	if (tech)
+		*tech = t;
+
+	return TRUE;
+}
+
+gboolean at_util_parse_reg(GAtResult *result, const char *prefix,
+				int *mode, int *status,
+				int *lac, int *ci, int *tech)
+{
+	GAtResultIter iter;
+	int m, s;
+	int l = -1, c = -1, t = -1;
+	const char *str;
+
+	g_at_result_iter_init(&iter, result);
+
+	while (g_at_result_iter_next(&iter, prefix)) {
+		g_at_result_iter_next_number(&iter, &m);
+
+		/* Sometimes we get an unsolicited CREG/CGREG here, skip it */
+		if (g_at_result_iter_next_number(&iter, &s) == FALSE)
+			continue;
+
+		if (g_at_result_iter_next_string(&iter, &str) == TRUE)
+			l = strtol(str, NULL, 16);
+		else
+			goto out;
+
+		if (g_at_result_iter_next_string(&iter, &str) == TRUE)
+			c = strtol(str, NULL, 16);
+		else
+			goto out;
+
+		g_at_result_iter_next_number(&iter, &t);
+
+out:
+		if (mode)
+			*mode = m;
+
+		if (status)
+			*status = s;
+
+		if (lac)
+			*lac = l;
+
+		if (ci)
+			*ci = c;
+
+		if (tech)
+			*tech = t;
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
