@@ -3,8 +3,6 @@
  *
  * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
  *
- * Contact: Alexander Kanavin <alexander.kanavin@nokia.com>
- *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * version 2 as published by the Free Software Foundation.
@@ -41,7 +39,8 @@
 #include <ofono/call-barring.h>
 #include "util.h"
 
-#include "isi.h"
+#include "isimodem.h"
+#include "isiutil.h"
 #include "ss.h"
 
 struct barr_data {
@@ -233,7 +232,7 @@ static bool query_resp_cb(GIsiClient *client, const void *restrict data,
 				if (!g_isi_sb_iter_get_byte(&iter, &bsc, 3 + i))
 					goto error;
 
-			        update_status_mask(&mask, bsc);
+				update_status_mask(&mask, bsc);
 			}
 			break;
 		}
