@@ -2,7 +2,7 @@
  *
  *  AT chat library with GLib integration
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -555,7 +555,7 @@ static void have_pdu(GAtChat *p, char *pdu)
 	gboolean listing_pdu = FALSE;
 
 	if (!pdu)
-		goto err;
+		goto error;
 
 	result.lines = g_slist_prepend(NULL, p->pdu_notify);
 	result.final_or_pdu = pdu;
@@ -580,7 +580,7 @@ static void have_pdu(GAtChat *p, char *pdu)
 
 	g_slist_free(result.lines);
 
-err:
+error:
 	g_free(p->pdu_notify);
 	p->pdu_notify = NULL;
 
@@ -1192,13 +1192,18 @@ gboolean g_at_chat_cancel(GAtChat *chat, guint id)
 	if (chat == NULL || chat->command_queue == NULL)
 		return FALSE;
 
+	/* We use id 0 for wakeup commands */
+	if (id == 0)
+		return FALSE;
+
 	l = g_queue_find_custom(chat->command_queue, GUINT_TO_POINTER(id),
 				at_command_compare_by_id);
 
 	if (!l)
 		return FALSE;
 
-	if (l == g_queue_peek_head(chat->command_queue)) {
+	if (l == g_queue_peek_head(chat->command_queue) &&
+			chat->cmd_bytes_written > 0) {
 		struct at_command *c = l->data;
 
 		/* We can't actually remove it since it is most likely
@@ -1214,6 +1219,33 @@ gboolean g_at_chat_cancel(GAtChat *chat, guint id)
 	return TRUE;
 }
 
+gboolean g_at_chat_cancel_all(GAtChat *chat)
+{
+	int n = 0;
+	struct at_command *c;
+
+	if (chat == NULL || chat->command_queue == NULL)
+		return FALSE;
+
+	while ((c = g_queue_peek_nth(chat->command_queue, n)) != NULL) {
+		if (c->id == 0) {
+			n += 1;
+			continue;
+		}
+
+		if (n == 0 && chat->cmd_bytes_written > 0) {
+			c->callback = NULL;
+			n += 1;
+			continue;
+		}
+
+		at_command_destroy(c);
+		g_queue_remove(chat->command_queue, c);
+	}
+
+	return TRUE;
+}
+
 static struct at_notify *at_notify_create(GAtChat *chat, const char *prefix,
 						gboolean pdu)
 {
@@ -1314,6 +1346,34 @@ gboolean g_at_chat_unregister(GAtChat *chat, guint id)
 	return TRUE;
 }
 
+gboolean g_at_chat_unregister_all(GAtChat *chat)
+{
+	GHashTableIter iter;
+	struct at_notify *notify;
+	char *prefix;
+	gpointer key, value;
+	GSList *l;
+
+	if (chat == NULL || chat->notify_list == NULL)
+		return FALSE;
+
+	g_hash_table_iter_init(&iter, chat->notify_list);
+
+	while (g_hash_table_iter_next(&iter, &key, &value)) {
+		prefix = key;
+		notify = value;
+
+		for (l = notify->nodes; l; l = l->next)
+			at_notify_node_destroy(l->data);
+
+		g_slist_free(notify->nodes);
+		notify->nodes = NULL;
+		g_hash_table_iter_remove(&iter);
+	}
+
+	return TRUE;
+}
+
 gboolean g_at_chat_set_wakeup_command(GAtChat *chat, const char *cmd,
 					unsigned int timeout, unsigned int msec)
 {
