@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -38,6 +38,7 @@
 
 #include "common.h"
 #include "storage.h"
+#include "idmap.h"
 
 #define DATA_CONNECTION_MANAGER_INTERFACE "org.ofono.DataConnectionManager"
 #define DATA_CONTEXT_INTERFACE "org.ofono.PrimaryDataContext"
@@ -70,10 +71,9 @@ struct ofono_gprs {
 	int cellid;
 	int technology;
 	int flags;
-	int next_context_id;
-	unsigned int cid_min;
-	unsigned int cid_max;
-	unsigned char *cid_bitmap;
+	struct idmap *pid_map;
+	unsigned int last_context_id;
+	struct idmap *cid_map;
 	int netreg_status;
 	struct ofono_netreg *netreg;
 	unsigned int netreg_watch;
@@ -108,6 +108,7 @@ struct pri_context {
 	ofono_bool_t active;
 	enum gprs_context_type type;
 	char name[MAX_CONTEXT_NAME_LENGTH + 1];
+	unsigned int id;
 	char *path;
 	char *key;
 	struct context_settings *settings;
@@ -145,44 +146,12 @@ static enum gprs_context_type gprs_context_string_to_type(const char *str)
 
 static unsigned int gprs_cid_alloc(struct ofono_gprs *gprs)
 {
-	unsigned int i, j;
-	unsigned int num_cids = gprs->cid_max - gprs->cid_min + 1;
-	unsigned int offset = num_cids / 8;
-	unsigned int max_bit = num_cids % 8;
-
-	for (i = 0; i < offset; i++) {
-		for (j = 0; j < 8; j++) {
-			if (gprs->cid_bitmap[i] & (1 << j))
-				continue;
-
-			gprs->cid_bitmap[i] |= (1 << j);
-			return i * 8 + j + 1;
-		}
-	}
-
-	for (j = 0; j < max_bit; j++) {
-		if (gprs->cid_bitmap[offset] & (1 << j))
-			continue;
-
-		gprs->cid_bitmap[i] |= (1 << j);
-		return offset * 8 + j + 1;
-	}
-
-	return 0;
+	return idmap_alloc(gprs->cid_map);
 }
 
 static void gprs_cid_release(struct ofono_gprs *gprs, unsigned int id)
 {
-	unsigned int offset = (id - 1) / 8;
-	unsigned int bit = (id - 1) % 8;
-
-	if (id > gprs->cid_max)
-		return;
-
-	if (id < gprs->cid_min)
-		return;
-
-	gprs->cid_bitmap[offset] &= ~(1 << bit);
+	idmap_put(gprs->cid_map, id);
 }
 
 static struct pri_context *gprs_context_by_path(struct ofono_gprs *gprs,
@@ -683,6 +652,11 @@ static DBusMessage *pri_set_property(DBusConnection *conn,
 	if (g_str_equal(property, "Active")) {
 		struct ofono_gprs_context *gc = ctx->gprs->context_driver;
 
+		if (gc == NULL || gc->driver->activate_primary == NULL ||
+				gc->driver->deactivate_primary == NULL ||
+				ctx->gprs->cid_map == NULL)
+			return __ofono_error_not_implemented(msg);
+
 		if (gc->pending)
 			return __ofono_error_busy(msg);
 
@@ -700,17 +674,14 @@ static DBusMessage *pri_set_property(DBusConnection *conn,
 		if (ctx->gprs->flags & GPRS_FLAG_ATTACHING)
 			return __ofono_error_attach_in_progress(msg);
 
-		if (gc == NULL || gc->driver->activate_primary == NULL ||
-				gc->driver->deactivate_primary == NULL)
-			return __ofono_error_not_implemented(msg);
-
 		if (value) {
 			ctx->context.cid = gprs_cid_alloc(ctx->gprs);
 
 			if (ctx->context.cid == 0)
 				return __ofono_error_failed(msg);
 
-			if (ctx->context.cid != 1) {
+			if (ctx->context.cid !=
+					idmap_get_min(ctx->gprs->cid_map)) {
 				ofono_error("Multiple active contexts are"
 						" not yet supported");
 
@@ -823,17 +794,17 @@ static gboolean context_dbus_register(struct pri_context *ctx)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
 	char path[256];
-	unsigned int id = ctx->gprs->next_context_id;
 	const char *basepath;
 
 	basepath = __ofono_atom_get_path(ctx->gprs->atom);
 
-	snprintf(path, sizeof(path), "%s/primarycontext%u", basepath, id);
+	snprintf(path, sizeof(path), "%s/primarycontext%u", basepath, ctx->id);
 
 	if (!g_dbus_register_interface(conn, path, DATA_CONTEXT_INTERFACE,
 					context_methods, context_signals,
 					NULL, ctx, pri_context_destroy)) {
 		ofono_error("Could not register PrimaryContext %s", path);
+		idmap_put(ctx->gprs->pid_map, ctx->id);
 		pri_context_destroy(ctx);
 
 		return FALSE;
@@ -842,8 +813,6 @@ static gboolean context_dbus_register(struct pri_context *ctx)
 	ctx->path = g_strdup(path);
 	ctx->key = ctx->path + strlen(basepath) + 1;
 
-	ctx->gprs->next_context_id += 1;
-
 	return TRUE;
 }
 
@@ -853,6 +822,7 @@ static gboolean context_dbus_unregister(struct pri_context *ctx)
 	char path[256];
 
 	strcpy(path, ctx->path);
+	idmap_put(ctx->gprs->pid_map, ctx->id);
 
 	return g_dbus_unregister_interface(conn, path,
 						DATA_CONTEXT_INTERFACE);
@@ -1222,6 +1192,7 @@ static DBusMessage *gprs_create_context(DBusConnection *conn,
 	const char *path;
 	enum gprs_context_type type;
 	char **objpath_list;
+	unsigned int id;
 
 	if (!dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &name,
 					DBUS_TYPE_STRING, &typestr,
@@ -1236,10 +1207,16 @@ static DBusMessage *gprs_create_context(DBusConnection *conn,
 	if (type == GPRS_CONTEXT_TYPE_INVALID)
 		return __ofono_error_invalid_format(msg);
 
-	if (g_slist_length(gprs->contexts) >= MAX_CONTEXTS)
+	if (gprs->last_context_id)
+		id = idmap_alloc_next(gprs->pid_map, gprs->last_context_id);
+	else
+		id = idmap_alloc(gprs->pid_map);
+
+	if (id > idmap_get_max(gprs->pid_map))
 		return __ofono_error_not_supported(msg);
 
 	context = pri_context_create(gprs, name, type);
+	context->id = id;
 
 	if (!context) {
 		ofono_error("Unable to allocate context struct");
@@ -1253,6 +1230,8 @@ static DBusMessage *gprs_create_context(DBusConnection *conn,
 		return __ofono_error_failed(msg);
 	}
 
+	gprs->last_context_id = id;
+
 	if (gprs->settings) {
 		g_key_file_set_string(gprs->settings, context->key,
 					"Name", context->name);
@@ -1455,11 +1434,10 @@ void ofono_gprs_set_cid_range(struct ofono_gprs *gprs,
 	if (gprs == NULL)
 		return;
 
-	g_free(gprs->cid_bitmap);
-	gprs->cid_bitmap = g_new0(unsigned char, (max - min + 8) / 8);
+	if (gprs->cid_map)
+		idmap_free(gprs->cid_map);
 
-	gprs->cid_min = min;
-	gprs->cid_max = max;
+	gprs->cid_map = idmap_new_from_range(min, max);
 }
 
 static void gprs_context_unregister(struct ofono_atom *atom)
@@ -1644,6 +1622,11 @@ static void gprs_unregister(struct ofono_atom *atom)
 
 	g_slist_free(gprs->contexts);
 
+	if (gprs->cid_map) {
+		idmap_free(gprs->cid_map);
+		gprs->cid_map = NULL;
+	}
+
 	if (gprs->netreg_watch) {
 		if (gprs->status_watch) {
 			__ofono_netreg_remove_status_watch(gprs->netreg,
@@ -1670,6 +1653,11 @@ static void gprs_remove(struct ofono_atom *atom)
 	if (gprs == NULL)
 		return;
 
+	if (gprs->pid_map) {
+		idmap_free(gprs->pid_map);
+		gprs->pid_map = NULL;
+	}
+
 	if (gprs->context_driver) {
 		gprs->context_driver->gprs = NULL;
 		gprs->context_driver = NULL;
@@ -1717,7 +1705,7 @@ struct ofono_gprs *ofono_gprs_create(struct ofono_modem *modem,
 	gprs->technology = -1;
 	gprs->cellid = -1;
 	gprs->location = -1;
-	gprs->next_context_id = 1;
+	gprs->pid_map = idmap_new(MAX_CONTEXTS);
 
 	return gprs;
 }
@@ -1752,6 +1740,13 @@ static gboolean load_context(struct ofono_gprs *gprs, const char *group)
 	gboolean ret = FALSE;
 	struct pri_context *context;
 	enum gprs_context_type type;
+	unsigned int id;
+
+	if (sscanf(group, "primarycontext%d", &id) != 1)
+		goto error;
+
+	if (id < 1 || id > MAX_CONTEXTS)
+		goto error;
 
 	if ((name = g_key_file_get_string(gprs->settings, group,
 					"Name", NULL)) == NULL)
@@ -1799,6 +1794,8 @@ static gboolean load_context(struct ofono_gprs *gprs, const char *group)
 	if ((context = pri_context_create(gprs, name, type)) == NULL)
 		goto error;
 
+	idmap_take(gprs->pid_map, id);
+	context->id = id;
 	strcpy(context->context.username, username);
 	strcpy(context->context.password, password);
 	strcpy(context->context.apn, apn);
@@ -1806,6 +1803,8 @@ static gboolean load_context(struct ofono_gprs *gprs, const char *group)
 	if (context_dbus_register(context) == FALSE)
 		goto error;
 
+	gprs->last_context_id = id;
+
 	gprs->contexts = g_slist_append(gprs->contexts, context);
 	ret = TRUE;
 
