@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -23,10 +23,13 @@
 
 #define OFONO_API_SUBJECT_TO_CHANGE
 
+void __ofono_exit();
+
 int __ofono_manager_init();
 void __ofono_manager_cleanup();
 
 const char **__ofono_modem_get_list();
+void __ofono_modem_shutdown();
 
 #include <ofono/log.h>
 
@@ -87,8 +90,9 @@ void __ofono_plugin_cleanup(void);
 
 #include <ofono/modem.h>
 
-unsigned int __ofono_modem_alloc_callid(struct ofono_modem *modem);
-void __ofono_modem_release_callid(struct ofono_modem *modem, int id);
+unsigned int __ofono_modem_callid_next(struct ofono_modem *modem);
+void __ofono_modem_callid_hold(struct ofono_modem *modem, int id);
+void __ofono_modem_callid_release(struct ofono_modem *modem, int id);
 
 struct ofono_atom;
 
@@ -163,11 +167,15 @@ void __ofono_atom_free(struct ofono_atom *atom);
 #include <ofono/devinfo.h>
 #include <ofono/phonebook.h>
 #include <ofono/sms.h>
-#include <ofono/sim.h>
 #include <ofono/voicecall.h>
 #include <ofono/gprs.h>
 #include <ofono/gprs-context.h>
 
+#include <ofono/sim.h>
+
+void __ofono_cbs_sim_download(struct ofono_sim *sim,
+				const guint8 *pdu, int pdu_len);
+
 #include <ofono/ssn.h>
 
 typedef void (*ofono_ssn_mo_notify_cb)(int index, void *user);
